<?php

namespace EventPortal\Domain\Component;

use Illuminate\Support\Arr;
use Illuminate\Support\Str;

abstract class Component
{
    /** @var string */
    public $type;

    /** @var string */
    public $name;

    /** @var string */
    public $label;

    /** @var null|string */
    public $classes;

    public function __construct(string $label)
    {
        $this->label = $label;
        $this->name = Str::slug($label, '_');
        $this->type = $this->type();
    }

    public static function make(string $label): self
    {
        return new static($label);
    }

    public function label(string $label): self
    {
        $this->label = $label;

        return $this;
    }

    /**
     * @param string|array|null $classes
     */
    public function classes($classes): self
    {
        $this->classes = Arr::wrap($classes);

        return $this;
    }

    /**
     * @param mixed $value
     * @return mixed|$this
     */
    public function when($value, callable $callback, ?callable $default = null)
    {
        if ($value) {
            return $callback($this, $value) ?: $this;
        }

        if ($default) {
            return $default($this, $value) ?: $this;
        }

        return $this;
    }

    /**
     * Returns the component as an array.
     */
    public function toArray(): array
    {
        $data = [
            'component' => $this->type,
            'name' => $this->name,
            'label' => $this->label,
            'clearable' => $this->clearable,
            'required' => $this->required,
        ];

        if ($this->placeholder !== null) {
            $data['placeholder'] = $this->placeholder;
        }

        if ($this->classes !== []) {
            $data['classes'] = $this->classes;
        }

        if ($this->value !== null) {
            $data['value'] = $this->value;
        }

        if ($this->validationRules !== null) {
            $data['validation'] = $this->validationRules;
        }

        return $data;
    }
}
